@@ -74,6 +74,13 @@
 /* XXX - should probably warn, here */
 #include "tslib/clutter-event-tslib.h"
 #endif
+#ifdef CLUTTER_INPUT_WAYLAND
+#include "wayland/clutter-device-manager-wayland.h"
+#endif
+
+#ifdef HAVE_CLUTTER_WAYLAND_COMPOSITOR
+#include <wayland-server.h>
+#endif
 
 G_DEFINE_ABSTRACT_TYPE (ClutterBackend, clutter_backend, G_TYPE_OBJECT);
 
@@ -105,6 +112,13 @@ enum
 
 static guint backend_signals[LAST_SIGNAL] = { 0, };
 
+/* Global for being able to specify a compositor side wayland display
+ * pointer before clutter initialization */
+#ifdef HAVE_CLUTTER_WAYLAND_COMPOSITOR
+static struct wl_display *_wayland_compositor_display;
+#endif
+
+
 static void
 clutter_backend_dispose (GObject *gobject)
 {
@@ -176,14 +190,10 @@ get_units_per_em (ClutterBackend       *backend,
       if (is_absolute)
         font_size = (gdouble) pango_size / PANGO_SCALE;
       else
-        font_size = (gdouble) pango_size / PANGO_SCALE
-                  * dpi
-                  / 96.0f;
+        font_size = dpi * ((gdouble) pango_size / PANGO_SCALE) / 72.0f;
 
       /* 10 points at 96 DPI is 13.3 pixels */
-      units_per_em = (1.2f * font_size)
-                   * dpi
-                   / 96.0f;
+      units_per_em = (1.2f * font_size) * dpi / 96.0f;
     }
   else
     units_per_em = -1.0f;
@@ -200,14 +210,20 @@ clutter_backend_real_resolution_changed (ClutterBackend *backend)
   ClutterBackendPrivate *priv = backend->priv;
   ClutterMainContext *context;
   ClutterSettings *settings;
+  gdouble resolution;
   gint dpi;
 
   settings = clutter_settings_get_default ();
   g_object_get (settings, "font-dpi", &dpi, NULL);
 
+  if (dpi < 0)
+    resolution = 96.0;
+  else
+    resolution = dpi / 1024.0;
+
   context = _clutter_context_get_default ();
   if (context->font_map != NULL)
-    cogl_pango_font_map_set_resolution (context->font_map, dpi / 1024.0);
+    cogl_pango_font_map_set_resolution (context->font_map, resolution);
 
   priv->units_per_em = get_units_per_em (backend, NULL);
   priv->units_serial += 1;
@@ -294,6 +310,11 @@ clutter_backend_real_create_context (ClutterBackend  *backend,
   if (backend->cogl_display == NULL)
     goto error;
 
+#ifdef HAVE_CLUTTER_WAYLAND_COMPOSITOR
+  cogl_wayland_display_set_compositor_display (backend->cogl_display,
+                                               _wayland_compositor_display);
+#endif
+
   CLUTTER_NOTE (BACKEND, "Setting up the display");
   if (!cogl_display_setup (backend->cogl_display, &internal_error))
     goto error;
@@ -420,22 +441,6 @@ clutter_backend_real_create_stage (ClutterBackend  *backend,
                        NULL);
 }
 
-static void
-clutter_backend_real_redraw (ClutterBackend *backend,
-                             ClutterStage   *stage)
-{
-  ClutterStageWindow *impl;
-
-  if (CLUTTER_ACTOR_IN_DESTRUCTION (stage))
-    return;
-
-  impl = _clutter_stage_get_window (stage);
-  if (impl == NULL)
-    return;
-
-  _clutter_stage_window_redraw (impl);
-}
-
 static void
 clutter_backend_real_init_events (ClutterBackend *backend)
 {
@@ -494,6 +499,14 @@ clutter_backend_real_init_events (ClutterBackend *backend)
       _clutter_events_tslib_init (backend);
     }
   else
+#endif
+#ifdef CLUTTER_INPUT_WAYLAND
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_WAYLAND) &&
+      (input_backend == NULL || input_backend == I_(CLUTTER_INPUT_WAYLAND)))
+    {
+      _clutter_events_wayland_init (backend);
+    }
+  else
 #endif
   if (input_backend != NULL)
     {
@@ -621,7 +634,6 @@ clutter_backend_class_init (ClutterBackendClass *klass)
   klass->ensure_context = clutter_backend_real_ensure_context;
   klass->get_features = clutter_backend_real_get_features;
   klass->create_stage = clutter_backend_real_create_stage;
-  klass->redraw = clutter_backend_real_redraw;
 }
 
 static void
@@ -703,28 +715,6 @@ _clutter_backend_create_stage (ClutterBackend  *backend,
   return stage_window;
 }
 
-void
-_clutter_backend_redraw (ClutterBackend *backend,
-                         ClutterStage   *stage)
-{
-  CLUTTER_STATIC_COUNTER (redraw_counter,
-                          "_clutter_backend_redraw counter",
-                          "Increments for each _clutter_backend_redraw call",
-                          0 /* no application private data */);
-  CLUTTER_STATIC_TIMER (redraw_timer,
-                        "Master Clock", /* parent */
-                        "Redrawing",
-                        "The time spent redrawing everything",
-                        0 /* no application private data */);
-
-  CLUTTER_COUNTER_INC (_clutter_uprof_context, redraw_counter);
-  CLUTTER_TIMER_START (_clutter_uprof_context, redraw_timer);
-
-  CLUTTER_BACKEND_GET_CLASS (backend)->redraw (backend, stage);
-
-  CLUTTER_TIMER_STOP (_clutter_uprof_context, redraw_timer);
-}
-
 gboolean
 _clutter_backend_create_context (ClutterBackend  *backend,
                                  GError         **error)
@@ -1084,6 +1074,9 @@ clutter_backend_get_resolution (ClutterBackend *backend)
   settings = clutter_settings_get_default ();
   g_object_get (settings, "font-dpi", &resolution, NULL);
 
+  if (resolution < 0)
+    return 96.0;
+
   return resolution / 1024.0;
 }
 
@@ -1291,3 +1284,28 @@ clutter_backend_get_cogl_context (ClutterBackend *backend)
 {
   return backend->cogl_context;
 }
+
+#ifdef HAVE_CLUTTER_WAYLAND_COMPOSITOR
+/**
+ * clutter_wayland_set_compositor_display:
+ * @display: A compositor side struct wl_display pointer
+ *
+ * This informs Clutter of your compositor side Wayland display
+ * object. This must be called before calling clutter_init().
+ *
+ * Since: 1.8
+ * Stability: unstable
+ */
+void
+clutter_wayland_set_compositor_display (struct wl_display *display)
+{
+  if (_clutter_context_is_initialized ())
+    {
+      g_warning ("%s() can only be used before calling clutter_init()",
+                 G_STRFUNC);
+      return;
+    }
+
+  _wayland_compositor_display = display;
+}
+#endif
