@@ -93,6 +93,9 @@
 
 #include <math.h>
 
+#define CLUTTER_DISABLE_DEPRECATION_WARNINGS
+#include "deprecated/clutter-container.h"
+
 #include "clutter-actor.h"
 #include "clutter-animatable.h"
 #include "clutter-bin-layout.h"
@@ -335,15 +338,16 @@ clutter_bin_layout_get_preferred_width (ClutterLayoutManager *manager,
                                         gfloat               *min_width_p,
                                         gfloat               *nat_width_p)
 {
-  GList *children = clutter_container_get_children (container);
-  GList *l;
+  ClutterActor *actor = CLUTTER_ACTOR (container);
+  ClutterActor *child;
   gfloat min_width, nat_width;
 
   min_width = nat_width = 0.0;
 
-  for (l = children; l != NULL; l = l->next)
+  for (child = clutter_actor_get_first_child (actor);
+       child != NULL;
+       child = clutter_actor_get_next_sibling (child))
     {
-      ClutterActor *child = l->data;
       gfloat minimum, natural;
 
       clutter_actor_get_preferred_width (child, for_height,
@@ -359,8 +363,6 @@ clutter_bin_layout_get_preferred_width (ClutterLayoutManager *manager,
 
   if (nat_width_p)
     *nat_width_p = nat_width;
-
-  g_list_free (children);
 }
 
 static void
@@ -370,15 +372,16 @@ clutter_bin_layout_get_preferred_height (ClutterLayoutManager *manager,
                                          gfloat               *min_height_p,
                                          gfloat               *nat_height_p)
 {
-  GList *children = clutter_container_get_children (container);
-  GList *l;
+  ClutterActor *actor = CLUTTER_ACTOR (container);
+  ClutterActor *child;
   gfloat min_height, nat_height;
 
   min_height = nat_height = 0.0;
 
-  for (l = children; l != NULL; l = l->next)
+  for (child = clutter_actor_get_first_child (actor);
+       child != NULL;
+       child = clutter_actor_get_next_sibling (child))
     {
-      ClutterActor *child = l->data;
       gfloat minimum, natural;
 
       clutter_actor_get_preferred_height (child, for_width,
@@ -394,8 +397,6 @@ clutter_bin_layout_get_preferred_height (ClutterLayoutManager *manager,
 
   if (nat_height_p)
     *nat_height_p = nat_height;
-
-  g_list_free (children);
 }
 
 static gdouble
@@ -426,17 +427,19 @@ clutter_bin_layout_allocate (ClutterLayoutManager   *manager,
                              const ClutterActorBox  *allocation,
                              ClutterAllocationFlags  flags)
 {
-  GList *children = clutter_container_get_children (container);
-  GList *l;
   gfloat allocation_x, allocation_y;
   gfloat available_w, available_h;
+  ClutterActor *actor, *child;
 
   clutter_actor_box_get_origin (allocation, &allocation_x, &allocation_y);
   clutter_actor_box_get_size (allocation, &available_w, &available_h);
 
-  for (l = children; l != NULL; l = l->next)
+  actor = CLUTTER_ACTOR (container);
+
+  for (child = clutter_actor_get_first_child (actor);
+       child != NULL;
+       child = clutter_actor_get_next_sibling (child))
     {
-      ClutterActor *child = l->data;
       ClutterLayoutMeta *meta;
       ClutterBinLayer *layer;
       ClutterActorBox child_alloc = { 0, };
@@ -471,8 +474,6 @@ clutter_bin_layout_allocate (ClutterLayoutManager   *manager,
                                          x_fill, y_fill,
                                          flags);
     }
-
-  g_list_free (children);
 }
 
 static GType
