@@ -58,9 +58,6 @@ struct _ClutterDesaturateEffect
 {
   ClutterOffscreenEffect parent_instance;
 
-  /* a back pointer to our actor, so that we can query it */
-  ClutterActor *actor;
-
   /* the desaturation factor, also known as "strength" */
   gdouble factor;
 
@@ -127,10 +124,6 @@ clutter_desaturate_effect_pre_paint (ClutterEffect *effect)
   if (!clutter_actor_meta_get_enabled (CLUTTER_ACTOR_META (effect)))
     return FALSE;
 
-  self->actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (effect));
-  if (self->actor == NULL)
-    return FALSE;
-
   if (!clutter_feature_available (CLUTTER_FEATURE_SHADERS_GLSL))
     {
       /* if we don't have support for GLSL shaders then we
@@ -364,8 +357,7 @@ clutter_desaturate_effect_set_factor (ClutterDesaturateEffect *effect,
     {
       effect->factor = factor;
 
-      if (effect->actor != NULL)
-        clutter_actor_queue_redraw (effect->actor);
+      clutter_effect_queue_repaint (CLUTTER_EFFECT (effect));
 
       g_object_notify_by_pspec (G_OBJECT (effect), obj_props[PROP_FACTOR]);
     }
