@@ -54,6 +54,20 @@
  * parented and exist between the emission of #ClutterDragAction::drag-begin
  * and #ClutterDragAction::drag-end.
  *
+ * <example id="drag-action-example">
+ *   <title>A simple draggable actor</title>
+ *   <programlisting>
+ * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../tests/interactive/test-drag.c">
+ *   <xi:fallback>FIXME: MISSING XINCLUDE CONTENT</xi:fallback>
+ * </xi:include>
+ *   </programlisting>
+ *   <para>The example program above allows dragging the rectangle around
+ *   the stage using a #ClutterDragAction. When pressing the
+ *   <keycap>Shift</keycap> key the actor that is going to be dragged is a
+ *   separate rectangle, and when the drag ends, the original rectangle will
+ *   be animated to the final coordinates.</para>
+ * </example>
+ *
  * #ClutterDragAction is available since Clutter 1.4
  */
 
@@ -78,6 +92,7 @@ struct _ClutterDragActionPrivate
   ClutterActor *drag_handle;
   ClutterDragAxis drag_axis;
 
+  ClutterInputDevice *device;
   gulong button_press_id;
   gulong capture_id;
 
@@ -87,6 +102,8 @@ struct _ClutterDragActionPrivate
 
   gfloat last_motion_x;
   gfloat last_motion_y;
+  ClutterModifierType last_motion_state;
+  ClutterInputDevice *last_motion_device;
 
   gfloat transformed_press_x;
   gfloat transformed_press_y;
@@ -164,7 +181,14 @@ emit_drag_begin (ClutterDragAction *action,
   ClutterDragActionPrivate *priv = action->priv;
 
   if (priv->stage != NULL)
-    _clutter_stage_set_motion_events_enabled (priv->stage, FALSE);
+    {
+      clutter_stage_set_motion_events_enabled (priv->stage, FALSE);
+      _clutter_stage_add_drag_actor (priv->stage,
+                                     clutter_event_get_device (event),
+                                     priv->drag_handle != NULL
+                                       ? priv->drag_handle
+                                       : actor);
+    }
 
   g_signal_emit (action, drag_signals[DRAG_BEGIN], 0,
                  actor,
@@ -183,6 +207,8 @@ emit_drag_motion (ClutterDragAction *action,
   gfloat motion_x, motion_y;
 
   clutter_event_get_coords (event, &priv->last_motion_x, &priv->last_motion_y);
+  priv->last_motion_state = clutter_event_get_state (event);
+  priv->last_motion_device = clutter_event_get_device (event);
 
   if (priv->drag_handle != NULL && !priv->emit_delayed_press)
     drag_handle = priv->drag_handle;
@@ -246,14 +272,23 @@ emit_drag_end (ClutterDragAction *action,
 {
   ClutterDragActionPrivate *priv = action->priv;
 
-  clutter_event_get_coords (event, &priv->last_motion_x, &priv->last_motion_y);
+  /* if we have an event, update our own state, otherwise we'll
+   * just use the currently stored state when emitting the ::drag-end
+   * signal
+   */
+  if (event != NULL)
+    {
+      clutter_event_get_coords (event, &priv->last_motion_x, &priv->last_motion_y);
+      priv->last_motion_state = clutter_event_get_state (event);
+      priv->last_motion_device = clutter_event_get_device (event);
+    }
 
   /* we might not have emitted ::drag-begin yet */
   if (!priv->emit_delayed_press)
     g_signal_emit (action, drag_signals[DRAG_END], 0,
                    actor,
                    priv->last_motion_x, priv->last_motion_y,
-                   clutter_event_get_state (event));
+                   priv->last_motion_state);
 
   /* disconnect the capture */
   if (priv->capture_id != 0)
@@ -262,8 +297,11 @@ emit_drag_end (ClutterDragAction *action,
       priv->capture_id = 0;
     }
 
-  _clutter_stage_set_motion_events_enabled (priv->stage,
-                                            priv->motion_events_enabled);
+  clutter_stage_set_motion_events_enabled (priv->stage,
+                                           priv->motion_events_enabled);
+
+  if (priv->last_motion_device != NULL)
+    _clutter_stage_remove_drag_actor (priv->stage, priv->last_motion_device);
 
   priv->in_drag = FALSE;
 }
@@ -279,7 +317,10 @@ on_captured_event (ClutterActor      *stage,
   actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (action));
 
   if (!priv->in_drag)
-    return FALSE;
+    return CLUTTER_EVENT_PROPAGATE;
+
+  if (clutter_event_get_device (event) != priv->device)
+    return CLUTTER_EVENT_PROPAGATE;
 
   switch (clutter_event_type (event))
     {
@@ -306,14 +347,14 @@ on_captured_event (ClutterActor      *stage,
     case CLUTTER_ENTER:
     case CLUTTER_LEAVE:
       if (priv->in_drag)
-        return TRUE;
+        return CLUTTER_EVENT_STOP;
       break;
 
     default:
       break;
     }
 
-  return FALSE;
+  return CLUTTER_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -324,10 +365,10 @@ on_button_press (ClutterActor      *actor,
   ClutterDragActionPrivate *priv = action->priv;
 
   if (!clutter_actor_meta_get_enabled (CLUTTER_ACTOR_META (action)))
-    return FALSE;
+    return CLUTTER_EVENT_PROPAGATE;
 
   if (clutter_event_get_button (event) != 1)
-    return FALSE;
+    return CLUTTER_EVENT_PROPAGATE;
 
   if (priv->stage == NULL)
     priv->stage = CLUTTER_STAGE (clutter_actor_get_stage (actor));
@@ -335,6 +376,8 @@ on_button_press (ClutterActor      *actor,
   clutter_event_get_coords (event, &priv->press_x, &priv->press_y);
   priv->press_state = clutter_event_get_state (event);
 
+  priv->device = clutter_event_get_device (event);
+
   priv->last_motion_x = priv->press_x;
   priv->last_motion_y = priv->press_y;
 
@@ -345,7 +388,7 @@ on_button_press (ClutterActor      *actor,
                                        &priv->transformed_press_y);
 
   priv->motion_events_enabled =
-    _clutter_stage_get_motion_events_enabled (priv->stage);
+    clutter_stage_get_motion_events_enabled (priv->stage);
 
   if (priv->x_drag_threshold == 0 || priv->y_drag_threshold == 0)
     emit_drag_begin (action, actor, event);
@@ -357,7 +400,7 @@ on_button_press (ClutterActor      *actor,
                                              G_CALLBACK (on_captured_event),
                                              action);
 
-  return FALSE;
+  return CLUTTER_EVENT_PROPAGATE;
 }
 
 static void
@@ -371,18 +414,25 @@ clutter_drag_action_set_actor (ClutterActorMeta *meta,
       ClutterActor *old_actor;
 
       old_actor = clutter_actor_meta_get_actor (meta);
-
-      g_signal_handler_disconnect (old_actor, priv->button_press_id);
-
-      if (priv->capture_id != 0)
-        g_signal_handler_disconnect (old_actor, priv->capture_id);
-
+      if (old_actor != NULL)
+        g_signal_handler_disconnect (old_actor, priv->button_press_id);
+        
       priv->button_press_id = 0;
-      priv->capture_id = 0;
+    }
 
+  if (priv->capture_id != 0)
+    {
+      if (priv->stage != NULL)
+        g_signal_handler_disconnect (priv->stage, priv->capture_id);
+
+      priv->capture_id = 0;
       priv->stage = NULL;
     }
 
+  clutter_drag_action_set_drag_handle (CLUTTER_DRAG_ACTION (meta), NULL);
+
+  priv->in_drag = FALSE;
+
   if (actor != NULL)
     priv->button_press_id = g_signal_connect (actor, "button-press-event",
                                               G_CALLBACK (on_button_press),
@@ -503,10 +553,14 @@ clutter_drag_action_dispose (GObject *gobject)
       ClutterActor *actor;
 
       actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (gobject));
-      g_signal_handler_disconnect (actor, priv->button_press_id);
+      if (actor != NULL)
+        g_signal_handler_disconnect (actor, priv->button_press_id);
+
       priv->button_press_id = 0;
     }
 
+  clutter_drag_action_set_drag_handle (CLUTTER_DRAG_ACTION (gobject), NULL);
+
   G_OBJECT_CLASS (clutter_drag_action_parent_class)->dispose (gobject);
 }
 
@@ -828,12 +882,25 @@ clutter_drag_action_get_drag_threshold (ClutterDragAction *action,
     *y_threshold = y_res;
 }
 
+static void
+on_drag_handle_destroy (ClutterActor      *actor,
+                        ClutterDragAction *action)
+{
+  ClutterDragActionPrivate *priv = action->priv;
+
+  /* make sure we reset the state */
+  if (priv->in_drag)
+    emit_drag_end (action, actor, NULL);
+
+  priv->drag_handle = NULL;
+}
+
 /**
  * clutter_drag_action_set_drag_handle:
  * @action: a #ClutterDragAction
- * @handle: a #ClutterActor
+ * @handle: (allow-none): a #ClutterActor, or %NULL to unset
  *
- * Sets the actor to be used as the drag handle
+ * Sets the actor to be used as the drag handle.
  *
  * Since: 1.4
  */
@@ -844,15 +911,25 @@ clutter_drag_action_set_drag_handle (ClutterDragAction *action,
   ClutterDragActionPrivate *priv;
 
   g_return_if_fail (CLUTTER_IS_DRAG_ACTION (action));
-  g_return_if_fail (CLUTTER_IS_ACTOR (handle));
+  g_return_if_fail (handle == NULL || CLUTTER_IS_ACTOR (handle));
 
   priv = action->priv;
 
   if (priv->drag_handle == handle)
     return;
 
+  if (priv->drag_handle != NULL)
+    g_signal_handlers_disconnect_by_func (priv->drag_handle,
+                                          G_CALLBACK (on_drag_handle_destroy),
+                                          action);
+
   priv->drag_handle = handle;
 
+  if (priv->drag_handle != NULL)
+    g_signal_connect (priv->drag_handle, "destroy",
+                      G_CALLBACK (on_drag_handle_destroy),
+                      action);
+
   g_object_notify_by_pspec (G_OBJECT (action), drag_props[PROP_DRAG_HANDLE]);
 }
 
