@@ -100,10 +100,9 @@ void                    clutter_do_event                        (ClutterEvent *e
 /* Debug utility functions */
 gboolean                clutter_get_accessibility_enabled       (void);
 
-CLUTTER_AVAILABLE_IN_1_14
-void                    clutter_disable_accessibility           (void);
-
 /* Threading functions */
+void                    clutter_threads_enter                   (void);
+void                    clutter_threads_leave                   (void);
 void                    clutter_threads_set_lock_functions      (GCallback enter_fn,
                                                                  GCallback leave_fn);
 guint                   clutter_threads_add_idle                (GSourceFunc    func,
@@ -123,11 +122,6 @@ guint                   clutter_threads_add_timeout_full        (gint
 guint                   clutter_threads_add_repaint_func        (GSourceFunc    func,
                                                                  gpointer       data,
                                                                  GDestroyNotify notify);
-CLUTTER_AVAILABLE_IN_1_10
-guint                   clutter_threads_add_repaint_func_full   (ClutterRepaintFlags flags,
-                                                                 GSourceFunc    func,
-                                                                 gpointer       data,
-                                                                 GDestroyNotify notify);
 void                    clutter_threads_remove_repaint_func     (guint          handle_id);
 
 void                    clutter_grab_pointer                    (ClutterActor  *actor);
