@@ -83,12 +83,13 @@
 #include "clutter-box.h"
 
 #include "clutter-actor-private.h"
-#include "clutter-color.h"
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
 #include "clutter-marshal.h"
 #include "clutter-private.h"
 
+#define CLUTTER_BOX_GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_BOX, ClutterBoxPrivate))
+
 struct _ClutterBoxPrivate
 {
   ClutterLayoutManager *manager;
@@ -110,7 +111,7 @@ static GParamSpec *obj_props[PROP_LAST] = { NULL, };
 
 static const ClutterColor default_box_color = { 255, 255, 255, 255 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (ClutterBox, clutter_box, CLUTTER_TYPE_ACTOR)
+G_DEFINE_TYPE (ClutterBox, clutter_box, CLUTTER_TYPE_ACTOR);
 
 static inline void
 clutter_box_set_color_internal (ClutterBox         *box,
@@ -127,7 +128,6 @@ clutter_box_real_get_paint_volume (ClutterActor       *actor,
                                    ClutterPaintVolume *volume)
 {
   gboolean retval = FALSE;
-  ClutterActorIter iter;
   ClutterActor *child;
 
   /* if we have a background color, and an allocation, then we need to
@@ -144,8 +144,9 @@ clutter_box_real_get_paint_volume (ClutterActor       *actor,
   /* otherwise, union the paint volumes of our children, in case
    * any one of them decides to paint outside the parent's allocation
    */
-  clutter_actor_iter_init (&iter, actor);
-  while (clutter_actor_iter_next (&iter, &child))
+  for (child = clutter_actor_get_first_child (actor);
+       child != NULL;
+       child = clutter_actor_get_next_sibling (child))
     {
       const ClutterPaintVolume *child_volume;
 
@@ -161,6 +162,22 @@ clutter_box_real_get_paint_volume (ClutterActor       *actor,
   return retval;
 }
 
+static void
+clutter_box_real_pick (ClutterActor       *actor,
+                       const ClutterColor *pick)
+{
+  ClutterActor *child;
+
+  CLUTTER_ACTOR_CLASS (clutter_box_parent_class)->pick (actor, pick);
+
+  for (child = clutter_actor_get_first_child (actor);
+       child != NULL;
+       child = clutter_actor_get_next_sibling (child))
+    {
+      clutter_actor_paint (child);
+    }
+}
+
 static void
 clutter_box_set_property (GObject      *gobject,
                           guint         prop_id,
@@ -214,30 +231,16 @@ clutter_box_get_property (GObject    *gobject,
     }
 }
 
-static void
-clutter_box_real_destroy (ClutterActor *actor)
-{
-  ClutterActor *iter;
-
-  iter = clutter_actor_get_first_child (actor);
-  while (iter != NULL)
-    {
-      ClutterActor *next = clutter_actor_get_next_sibling (iter);
-
-      clutter_actor_destroy (iter);
-
-      iter = next;
-    }
-}
-
 static void
 clutter_box_class_init (ClutterBoxClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
-  actor_class->destroy = clutter_box_real_destroy;
+  g_type_class_add_private (klass, sizeof (ClutterBoxPrivate));
+
   actor_class->get_paint_volume = clutter_box_real_get_paint_volume;
+  actor_class->pick = clutter_box_real_pick;
 
   gobject_class->set_property = clutter_box_set_property;
   gobject_class->get_property = clutter_box_get_property;
@@ -278,7 +281,7 @@ clutter_box_class_init (ClutterBoxClass *klass)
 static void
 clutter_box_init (ClutterBox *self)
 {
-  self->priv = clutter_box_get_instance_private (self);
+  self->priv = CLUTTER_BOX_GET_PRIVATE (self);
 }
 
 /**
